/*
    Dies ist meine zentrale Lib.

    Diese Funktion ermittelt die Kalenderwoche.

    Copyright (C) 1998 Andreas Hofmeier

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


    Adresse:  Andreas Hofmeier
              Stralsunder Strae 20
              28239 Bremen
	      Germany

    Telefon:  0421-613793

    eMail: hofmeier@bfe.uni-bremen.de */


#include "hofmeier.h"
#include <stdio.h>
#include <time.h>


/* Giebt den ersten Montag des Jahres ti zurck. Alle angaben
   in Secunden von 00:00:00 am January 1, 1970, Coordinated
   Universal Time (UTC). */
unsigned long get_first_monday_of_the_year(unsigned long ti)
{
  struct tm *tminfo;
  int y;

  tminfo = localtime(&ti);
  y = tminfo -> tm_year;
  
  memset(tminfo, 0, sizeof(struct tm));

  tminfo -> tm_sec = 0;
  tminfo -> tm_min = 0;
  tminfo -> tm_hour = 12;
  tminfo -> tm_mday = 1;
  tminfo -> tm_mon = 0;
  tminfo -> tm_year = y;

  ti = mktime(tminfo);
  tminfo = localtime(&ti);
  for (  ; 1 != (tminfo -> tm_wday);  ) {
    ti += (24 * 60 * 60); // Ein Tag weiter...
    tminfo = localtime(&ti);
  }
  return ti;
}


/* Errechnet die KalendarWoche. Bekommt das Datum wessen Woche
   ermittelt werden soll. */
int getweek(unsigned long ti)
{
  struct tm *tminfo;
  struct tm *tminfo_first_mon;
  int first_monday_in_year_yday = 0;
  unsigned long first_mondey_of_the_year;
  unsigned long mondey_of_the_week;
  unsigned long diver;

  /* Acktueller Tag 00:00:00 */
  tminfo = localtime(&ti);
  tminfo -> tm_hour = 12;
  tminfo -> tm_min  = 0;
  tminfo -> tm_sec  = 0;
  ti = mktime(tminfo);

  /* Ermitteln des Ersten montages im Jahr */
  first_mondey_of_the_year = get_first_monday_of_the_year(ti);
  tminfo_first_mon = localtime(&first_mondey_of_the_year);
  first_monday_in_year_yday = tminfo_first_mon -> tm_yday;

  /* Wenn Aktuelles Datum kleiner als erster montag EXIT */
  if (ti < first_mondey_of_the_year) {
    return 1;
  }
  /* Montag der Woche */
  mondey_of_the_week = ti;
  tminfo = localtime(&mondey_of_the_week);
  for (  ; 1 != (tminfo -> tm_wday);  ) {
    mondey_of_the_week -= (24 * 60 * 60); /* Ein Tag zurck... */
    tminfo = localtime(&mondey_of_the_week);
  } 
  // Erste Tag im Jahr ist nicht Montag
  if ((first_monday_in_year_yday) > 0) {
    // erste +1: tm_yday begint bei 0
    // zweite +1: Vor dem Erstem Montag im Jahr ist noch eine Abgeschnittene Woche...
    // zweite +1: In der ersten Woche wird das Ergebniss (/7) kleiner als eins sein...
    return ((((tminfo -> tm_yday) - (first_monday_in_year_yday) + 1) / 7) + 2);
  } else { // Erstaer Tag im Jahr ist Montag
    // erste +1: tm_yday begint bei 0
    // zweite +1: In der ersten Woche wird das Ergebniss (/7) kleiner als eins sein...
    return ((((tminfo -> tm_yday)/* - (first_monday_in_year_yday)*/ + 1) / 7) + 1);
  }
}


